@@ -43,6 +43,7 @@ from geodataflow.core.schemadef import DataType, FieldDef
 from osgeo import gdalconst as gdal_const
 from osgeo import gdal_array
 from osgeo import osr
+from shapely.wkb import loads as shapely_wkb_loads, dumps as shapely_wkb_dumps
 osr.UseExceptions()
 
 # Default schema of a GDAL Dataset.
@@ -118,7 +119,9 @@ class GdalDataset:
         Returns the Properties of this Dataset.
         """
         info = self.get_metadata()
-        return {field_def.name: info.get(field_def.name) for field_def in DATASET_DEFAULT_SCHEMA_DEF}
+        data = {field_def.name: info.get(field_def.name) for field_def in DATASET_DEFAULT_SCHEMA_DEF}
+        data.update(self.user_data)
+        return data
 
     def env(self) -> GdalEnv:
         """
@@ -217,6 +220,10 @@ class GdalDataset:
         Returns the EPSG SRID code of the this Dataset.
         """
         spatial_wkt = self._dataset.GetProjection()
+
+        if not spatial_wkt:
+            return 0
+
         spatial_ref = osr.SpatialReference()
         spatial_ref.ImportFromWkt(spatial_wkt)
 
@@ -273,9 +280,50 @@ class GdalDataset:
         geometry = geometry.with_srid(info.get('srid'))
         return geometry
 
+    def _geometry_to_layer(self, geometry) -> str:
+        """
+        Create a FeatureLayer from the specified Geometry.
+        """
+        gdal_env = self.env()
+        ogr = gdal_env.ogr()
+
+        attribute_name = 'my_id'
+        layer_name = str(uuid.uuid4()).replace('-', '')
+        layer_file = os.path.join(gdal_env.temp_data_path(), '{}.shp'.format(layer_name))
+
+        spatial_ref = osr.SpatialReference()
+        spatial_ref.ImportFromEPSG(geometry.get_srid())
+        if hasattr(spatial_ref, 'SetAxisMappingStrategy'):
+            spatial_ref.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
+
+        driver = ogr.GetDriverByName('ESRI Shapefile')
+        feature_store = driver.CreateDataSource(layer_file, options=[])
+        feature_layer = feature_store.CreateLayer(layer_name, srs=spatial_ref, geom_type=ogr.wkbMultiPolygon)
+        feature_field = ogr.FieldDefn(attribute_name, ogr.OFTInteger)
+        feature_layer.CreateField(feature_field)
+        layer_schema = feature_layer.GetLayerDefn()
+
+        geometry_wkb = shapely_wkb_dumps(geometry)
+        geometry_ogr = ogr.CreateGeometryFromWkb(geometry_wkb)
+
+        # Saving the geometries of interest to a temporary Feature Layer to rasterize.
+        feature = ogr.Feature(layer_schema)
+        feature.SetField(attribute_name, 1)
+        feature.SetGeometry(geometry_ogr)
+        feature_layer.CreateFeature(feature)
+        feature = None
+
+        layer_schema = None
+        feature_field = None
+        feature_layer = None
+        feature_store = None
+        return layer_file
+
     def warp(self,
              output_crs=None, output_res_x: float = None, output_res_y: float = None, output_geom=None,
-             resample_arg: int = gdal_const.GRA_Bilinear) -> "GdalDataset":
+             resample_arg: int = gdal_const.GRA_Bilinear,
+             cutline: bool = False,
+             all_touched: bool = True) -> "GdalDataset":
         """
         Warp this Dataset by specified parameters.
         """
@@ -311,12 +359,22 @@ class GdalDataset:
 
             if output_res_x and output_res_y:
                 envelope = GeometryUtils.clamp_envelope(list(geometry.bounds), output_res_x, output_res_y)
+                geometry = GeometryUtils.create_geometry_from_bbox(*envelope).with_srid(geometry.get_srid())
             else:
                 envelope = list(geometry.bounds)
 
+            layer_file = \
+                self._geometry_to_layer(geometry) if cutline else None
+
+            warp_options = []
+
+            if cutline and all_touched:
+                warp_options.append("CUTLINE_ALL_TOUCHED")
+
             # Define the Warping options for resampling and reprojection.
             options = gdal.WarpOptions(format='Gtiff',
                                        outputBounds=envelope,
+                                       cutlineDSName=layer_file,
                                        resampleAlg=resample_arg,
                                        xRes=output_res_x,
                                        yRes=output_res_y,
@@ -325,6 +383,7 @@ class GdalDataset:
                                        dstSRS='EPSG:' + str(output_crs.to_epsg()),
                                        dstNodata=info.get('noData'),
                                        copyMetadata=True,
+                                       warpOptions=warp_options,
                                        creationOptions=['TILED=YES', 'COMPRESS=DEFLATE', 'PREDICTOR=2'])
         else:
             source_srid = info.get('srid')
@@ -389,7 +448,7 @@ class GdalDataset:
         command_line += ' --format Gtiff --co "TILED=YES" --co "COMPRESS=DEFLATE" --co "PREDICTOR=2"'
         gdal_env.run(command_line)
         #
-        return GdalDataset(calc_file, gdal_env, self.user_data.copy(), True)
+        return GdalDataset(calc_file, gdal_env, self.user_data.copy(), False)
 
     def split(self, tile_size_x: int, tile_size_y: int, padding: int = 0) -> Iterable["GdalDataset"]:
         """
@@ -467,6 +526,60 @@ class GdalDataset:
 
         pass
 
+    def polygonize(self, band_index: int = 0) -> Any:
+        """
+        Returns the Polygon of all connected regions of nodata pixels in the raster.
+        """
+        raster_count = self._dataset.RasterCount
+
+        if band_index >= raster_count:
+            raise Exception('Current Dataset only has {} Bands. You are defined the {} th one.'
+                            .format(raster_count, band_index))
+
+        gdal_env = self.env()
+        gdal = gdal_env.gdal()
+        ogr = gdal_env.ogr()
+
+        layer_name = str(uuid.uuid4()).replace('-', '')
+        layer_file = '/vsimem/{}'.format(layer_name)
+        srid = self.get_spatial_srid()
+
+        spatial_ref = osr.SpatialReference()
+        spatial_ref.ImportFromEPSG(srid)
+        if hasattr(spatial_ref, 'SetAxisMappingStrategy'):
+            spatial_ref.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)
+
+        driver = ogr.GetDriverByName('Memory')
+        feature_store = driver.CreateDataSource(layer_file, options=[])
+        feature_layer = feature_store.CreateLayer(layer_name, srs=spatial_ref, geom_type=ogr.wkbMultiPolygon)
+        feature_field = ogr.FieldDefn('value', ogr.OFTInteger)
+        feature_layer.CreateField(feature_field)
+
+        # Polygonize specified Band.
+        rs_band = self._dataset.GetRasterBand(band_index + 1)
+        ms_band = rs_band.GetMaskBand()
+        gdal.Polygonize(rs_band, ms_band, feature_layer, 0, [], callback=None)
+        ms_band = None
+        rs_band = None
+
+        # Get Union of valid Geometries (value != nan).
+        geometry = None
+        for feature in feature_layer:
+            value = feature.GetField(0)
+
+            if value != -2147483648:
+                geometry_i = feature.GetGeometryRef()
+                geometry_i = shapely_wkb_loads(bytes(geometry_i.ExportToWkb()))
+                geometry = geometry_i if geometry is None else geometry.union(geometry_i)
+
+            pass
+
+        feature_field = None
+        feature_layer = None
+        feature_store.Destroy()
+        feature_store = None
+        return geometry.with_srid(srid) if geometry else None
+
     @staticmethod
     def mosaic_of_datasets(datasets: List["GdalDataset"],
                            resample_arg: int = gdal_const.GRA_Bilinear) -> "GdalDataset":
@@ -493,6 +606,7 @@ class GdalDataset:
 
             if info.get('srid') == first_info.get('srid'):
                 mosaic_of_files.append(dataset.dataset_path(force_exists=True))
+                dataset._recyclable = False
             else:
                 transform_fn = GeometryUtils.create_transform_function(
                     info.get('srid'), first_info.get('srid')
